/**
 * RFID chip detector object for MaxMSP.
 */

import com.cycling74.max.*;
import iwork.eheap2.*;

public class mgRFIDDetect extends MaxObject implements Executable {
	
	String		nameEventHeap = "localhost";	// Name of the Event Heap
	
	int    		numInlets = 1;		    // Number of inlets
	float		updateInterval = 250;	// Time intervall
	String[]	listRFIDs;				// List of RFIDs
	
	EventHeap	eheap;					// Event heap
	Event		eventRFID;				// Template RFID event
	
	private MaxClock timer;		// Update timer

	// Constructor
	
	public mgRFIDDetect(Atom[] args) {
		post("mgRFIDDetect class created ...");
		
		// Retrieve name of event heap
		if(args.length == 0)
		{
			post("WARNING: First argument required to determine name of " +
			     "event heap. Using default (localhost).");
		}
		else
			nameEventHeap = args[0].getString();
		post("    Event heap name = " + nameEventHeap);

		// Retrieve number of RFID inlets
		if(args.length < 1)
		{
			post("WARNING: Second argument required to determine number of " +
			     "RFID inlets. Using default (1).");
		}
		else
		{
			numInlets = args[1].getInt();
			if(numInlets < 1) numInlets = 1;
		}
		post("    Number of RFID inlets = " + numInlets);
		
		// Retrieve time interval for updater
		if(args.length < 3)
		{
			post("WARNING: Third argument required to determine time " +
			     "interval for updating. Using default (250 ms).");
		}
		else
			updateInterval = args[2].getInt();
		post("    Update interval = " + updateInterval);

		// Declare the inlets and outlets
		int[] inletTypes  = new int[numInlets];
		int[] outletTypes = new int[numInlets-1];
		for(int i = 0; i < numInlets; i++)
			inletTypes[i]  = DataTypes.ALL;
		for(int i = 0; i < numInlets-1; i++)
			outletTypes[i] = DataTypes.ALL;
		declareInlets(inletTypes);
		declareOutlets(outletTypes);
		
		// Create array for RFIDs
		listRFIDs = new String[numInlets];
		
		// Create event heap
		eheap = new EventHeap(nameEventHeap);
		post("Event heap created.");
		
		// Create template event for RFID receiving
		try {		
			eventRFID = new Event("RFID");
		}
		catch(Exception ex) {
			ex.printStackTrace();
		}

		// Create and start timer
		timer = new MaxClock(this);
		timer.unset();
	}

	// Object is to be deleted
	
	protected void notifyDeleted() {
		// Release timer
		timer.unset();
	}

	// Inlet has been passed
		
	protected void anything(String message, Atom[] args) {
		// Store RFID string
		int index = getInlet();
		
		// Toggle timer on/off
		if(message.equals("int") && args.length > 0 && args[0].isInt())
		{
			if(1 == args[0].getInt())
			{
				timer.delay(updateInterval);
				post("mgRFIDDetect enabled.");
			}
			else
			{
				timer.unset();
				post("mgRFIDDetect disabled.");
			}
			
			bang();
		}
		// Store RFID
		else if(index >= 0 && index < listRFIDs.length)
		{
			// Skip leading underline
			if(message.length() > 0 && message.charAt(0) == '_')
				listRFIDs[index] = message.substring(1);
			else
				listRFIDs[index] = "";
		}
	}
	
	// Return help for inlets
                        
	public String getInletAssist(int inletIdx)
	{
		if(inletIdx == 0)
			return "Toggles update process on (1) or off (0)";
		else
			return "String to identify RFID chip (id tag)";
	}

	// Return help for outlets
                        
	public String getOutletAssist(int inletIdx)
	{
		return "Index of activated RFID tag (1 to n)";
	}

	// Bang
	
	public void bang() {
		/*post("mgRFIDDetect info");
		for(int i = 0; i < listRFIDs.length; i++)
			post("" + i + ": " + listRFIDs[i]);*/
	}
	
	// Execute update process

	public void execute() {		
		try {		
			// Check if new RFID tag is available
			Event event = eheap.getEvent(eventRFID);
			if(null != event)
			{
				// Get out event tag
				String tagID = event.getPostValueString("id");
				if(tagID.equals("NULL"))
					post("ERROR: Invalid tag id has been sent (NULL).");
				else
				{
					// Find this tag in our RFID list
					String out = "Received RFID tag '" + tagID + "': ";
					int i;
					for(i = 0; i < listRFIDs.length; i++)
					{
						if(null == listRFIDs[i])
							continue;
						
						if(listRFIDs[i].equals(tagID))
						{
							out += "found and send out.";
							
							// Band the corresponding outlet
							outletBang(i);
							break;
						}
					}
					if(i == listRFIDs.length)
						out += "dropped (not in list).";
					post(out);
				}
			}

			timer.delay(updateInterval);
		}
		catch(Exception ex) {
			ex.printStackTrace();
		}
	}
}
